/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * 
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * The Original Code is keywordManager.
 * 
 * The Initial Developer of the Original Code is 
 * Rgis Caspar. 
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s): 
 *   Cory Schmidt: keywordBar author
 *   http://developer.mozilla.org/en/docs/Places:Query_System author(s)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


// Constants are prefixed because Cc and Ci are used in controller.js
// and controller.js is loaded as a sub-script.
const kwm_Cc = Components.classes;
const kwm_Ci = Components.interfaces;


var KWMView = null;


function BM_NS(aProperty)
{
  return "http://home.netscape.com/NC-rdf#" + aProperty;
}


function $(aID)
{
  return document.getElementById(aID);
}


function Element(name, shortcut, bookmark)
{
  this.name = name;
  this.shortcut = shortcut;
  this.bookmark = bookmark;
}


function keywordmanager()
{
  this.elements = [];
  this.controllerLoaded = false;
  // *hack* to check if Places is enabled or not :)
  this.version = ('nsINavBookmarksService' in kwm_Ci) ? 2 : 1;
}


keywordmanager.prototype.init = function()
{
  switch (this.version)
  {
    case 1: // RDF bookmarks
      // ----------------------------------
      //   borrowed from keywordBar-0.3.1  
      // ----------------------------------
      var dataSource = kwm_Cc["@mozilla.org/rdf/datasource;1?name=bookmarks"]
                       .getService(kwm_Ci.nsIRDFDataSource);
      var service = kwm_Cc["@mozilla.org/rdf/rdf-service;1"]
                    .getService(kwm_Ci.nsIRDFService);
      var resources = dataSource.GetAllResources();
      var sURLPredicate = service.GetResource( BM_NS("ShortcutURL") ); 
      var namePredicate = service.GetResource( BM_NS("Name") );
      
      while( resources.hasMoreElements() )
      {
        var bookmark = resources.getNext();
      
        if( bookmark instanceof kwm_Ci.nsIRDFResource )
        {
          var name = dataSource.GetTarget( bookmark, namePredicate, true );
          var shortcutURL = dataSource.GetTarget( bookmark, sURLPredicate, true );
         
          if( shortcutURL instanceof kwm_Ci.nsIRDFLiteral && 
              name instanceof kwm_Ci.nsIRDFLiteral )
            this.elements.push( new Element( name.Value, shortcutURL.Value, 
                                             bookmark.Value ) );
        }
      }
      
      break;

    case 2: // Places
      // --------------------------------------------------------------------------
      //   borrowed from http://developer.mozilla.org/en/docs/Places:Query_System
      // --------------------------------------------------------------------------
      var historyService = kwm_Cc["@mozilla.org/browser/nav-history-service;1"]
                           .getService(kwm_Ci.nsINavHistoryService);
      var bookmarkService = kwm_Cc["@mozilla.org/browser/nav-bookmarks-service;1"]
                            .getService(kwm_Ci.nsINavBookmarksService);
      // get all history sorted by ascending visit date
      var options = historyService.getNewQueryOptions();
      options.sortingMode = options.SORT_BY_TITLE_ASCENDING;
      options.setGroupingMode([options.GROUP_BY_FOLDER], 1);
      // no query parameters will return everything
      var query = historyService.getNewQuery();
      query.setFolders([bookmarkService.bookmarksRoot], 1);
      query.onlyBookmarked = true;
      var result = historyService.executeQuery(query, options);
      this.walk(result.root, this.elements);
      break;
      
    default:
      // nothing
  }
  
  
  // fallback if no elements found (actual Trunk behavior)
  if (this.elements.length == 0) 
  {
    this.version = 1;
    this.init();
    return;
  }
  
  // update tree
  KWMView =  new keywordmanagerView(this.elements);
  $("kwm-tree").view = KWMView;
  KWMView.cycleHeader( { id: "kwm-treecol-name" } );
}


keywordmanager.prototype.walk  = function(aNode, aElem)
{
  var aNode = aNode.QueryInterface(kwm_Ci.nsINavHistoryFolderResultNode);
  aNode.containerOpen = true;
  
  for (var i = 0; i < aNode.childCount; i++) 
  {
    var child = aNode.getChild(i);
  
    switch (child.type)
    {
      case aNode.RESULT_TYPE_FOLDER:
        this.walk(child, aElem);
        break;
      
      case aNode.RESULT_TYPE_SEPARATOR:
        break;
      
      default:
        var bms = kwm_Cc["@mozilla.org/browser/nav-bookmarks-service;1"]
                  .getService(kwm_Ci.nsINavBookmarksService);
        var ios = kwm_Cc["@mozilla.org/network/io-service;1"]
                  .getService(kwm_Ci.nsIIOService);
        var uri = ios.newURI(child.uri, null, null);
        var kw = bms.getKeywordForURI(uri);
        
        if (kw)
          aElem.push( new Element(child.title, kw, uri) );
    }
  }
  
  aNode.containerOpen = false;
}


keywordmanager.prototype.clearFilter = function()
{
  var fb = $("kwm-filter-box");
  fb.value = ""; 
  fb.focus();
  $("kwm-filter-button").setAttribute("disabled", "true");
  
  // update tree
  KWMView = new keywordmanagerView(KWM.elements);
  $("kwm-tree").view = KWMView;
  KWMView.invalidate();
}


keywordmanager.prototype.setFilter = function(aTextBox)
{
  var regexp;
  
  if (aTextBox.value == "")
  {
    KWM.clearFilter();
    return;
  }
  
  $("kwm-filter-button").setAttribute("disabled", "false"); 
  
  try 
  {
    regexp = new RegExp(aTextBox.value, "i");
  }
  catch (e)
  {
    // bad regexp
    return;
  }
  
  var elements = KWM.elements.filter(
    function(aElem, anIndex, anArray) 
    {
      return (aElem.name.match(regexp) || aElem.shortcut.match(regexp));
    }
  );
  
  // update tree
  KWMView = new keywordmanagerView(elements);
  $("kwm-tree").view = KWMView;
  KWMView.invalidate();
}


keywordmanager.prototype.destroy = function()
{
  KWM = null;
  KWMView = null;
}


keywordmanager.prototype.doubleclick = function(aEvt)
{
  var row = {}, col = {}, obj = {};
  var name, shortcut;
  $("kwm-tree").treeBoxObject.getCellAt(aEvt.clientX, aEvt.clientY, row, col, obj);
  
  if (obj.value == "text")
  {
    var elem = KWMView.selectedElement;
    
    // -------------------------------------------------------------------
    //   borrowed from 'chrome://browser/content/bookmarks/bookmarks.js'
    // -------------------------------------------------------------------
    if (KWM.version == 1)
    {
      openDialog( "chrome://browser/content/bookmarks/bookmarksProperties.xul", 
                  "", "centerscreen,chrome,modal,resizable=no", elem.bookmark, 
                  {} );
      var service = kwm_Cc["@mozilla.org/rdf/rdf-service;1"]
                    .getService(kwm_Ci.nsIRDFService);
      var resource = service.GetResource(elem.bookmark);
      var bmds = service.GetDataSource("rdf:bookmarks");
      var sURLPredicate = service.GetResource( BM_NS("ShortcutURL") ); 
      var namePredicate = service.GetResource( BM_NS("Name") );
      var name = bmds.GetTarget(resource, namePredicate, true);
      var shortcut = bmds.GetTarget(resource, sURLPredicate, true);
      name = name.QueryInterface(kwm_Ci.nsIRDFLiteral).Value;
      shortcut = shortcut.QueryInterface(kwm_Ci.nsIRDFLiteral).Value;
    } 
    else
    { 
      if (!KWM.controllerLoaded)
      {
        var loader =  kwm_Cc["@mozilla.org/moz/jssubscript-loader;1"]
                      .getService(kwm_Ci.mozIJSSubScriptLoader);
        loader.loadSubScript("chrome://browser/content/places/controller.js");
        KWM.controllerLoaded = true;
      }
      
      var bms = kwm_Cc["@mozilla.org/browser/nav-bookmarks-service;1"]
                .getService(kwm_Ci.nsINavBookmarksService);
      PlacesController.showBookmarkProperties(elem.bookmark);
      name = bms.getItemTitle(elem.bookmark);
      shortcut = bms.getKeywordForURI(elem.bookmark);
    }
    
    // update tree
    elem.name = name;
    elem.shortcut = shortcut;
    KWMView.invalidate();
  } 
}


var KWM = new keywordmanager();

