/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * 
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * The Original Code is keywordManager.
 * 
 * The Initial Developer of the Original Code is 
 * Rgis Caspar. 
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s): 
 *   Gavin Sharp & Ben Goodger (engineManager code)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */
 
// ----------------------------------
//   borrowed from engineManager.js
// ----------------------------------

function keywordmanagerView(elements) {
  this._elements = elements;
}

keywordmanagerView.prototype = {
  _elements: null,
  _sortOrder: { name: 1, keyword: 1 },
  tree: null,

  get lastIndex() {
    return this.rowCount - 1;
  },
  
  get selectedIndex() {
    var seln = this.selection;
    if (seln.getRangeCount() > 0) {
      var min = { };
      seln.getRangeAt(0, min, { });
      return min.value;
    }
    return -1;
  },
  
  get selectedElement() {
    return this._elements[this.selectedIndex];
  },

  // Helpers
  rowCountChanged: function (index, count) {
    this.tree.rowCountChanged(index, count);
  },

  invalidate: function () {
    this.tree.invalidate();
  },

  ensureRowIsVisible: function (index) {
    this.tree.ensureRowIsVisible(index);
  },

  // nsITreeView
  get rowCount() {
    return this._elements.length;
  },
  
  cycleHeader: function(aColumn) {
    var order;
    var col1, col2;
   
    if (aColumn.id == "kwm-treecol-name")
    {
      this._elements = this._elements.sort(
        function (a, b) {
          if (a.name < b.name)
            return -1;
          if (a.name > b.name)
            return 1;
          return 0;
        }
      );
      
      if (this._sortOrder.name < 0)
        this._elements = this._elements.reverse();
      
      col1 = document.getElementById("kwm-treecol-name");
      col2 = document.getElementById("kwm-treecol-keyword");
      order = (this._sortOrder.name < 0) ? "a" : "de";
      this._sortOrder.name = -this._sortOrder.name;
      this._sortOrder.keyword = 1;
    } 
    
    if (aColumn.id == "kwm-treecol-keyword")
    {
      
      this._elements = this._elements.sort(
        function (a, b) {
          if (a.shortcut < b.shortcut)
            return -1;
          if (a.shortcut > b.shortcut)
            return 1;
          return 0;
        }
      );
      
      if (this._sortOrder.keyword < 0)
        this._elements = this._elements.reverse();
      
      col1 = document.getElementById("kwm-treecol-keyword");
      col2 = document.getElementById("kwm-treecol-name");
      order = (this._sortOrder.keyword < 0) ? "a" : "de";
      this._sortOrder.keyword = -this._sortOrder.keyword;
      this._sortOrder.name = 1;
    }
    
    if (col1 && col2 && order)
    {
      col1.setAttribute("sortActive", "true");
      col1.setAttribute("sortDirection", order + "scending");
      col2.setAttribute("sortActive", "false");
      col2.setAttribute("sortDirection", "natural");
    }
  },
  
  getCellText: function(index, column) {
    if (column.id == "kwm-treecol-name")
      return this._elements[index].name;
    if (column.id == "kwm-treecol-keyword")
      return this._elements[index].shortcut;
    return "";
  },

  setTree: function(tree) {
    this.tree = tree;
  },

  getImageSrc: function(index, column) { return ""; },
  drop: function(newIndex, orientation) { },
  canDrop: function(targetIndex, orientation) { return false; },
  selection: null,
  isSorted: function(index) { return false; },
  getRowProperties: function(index, properties) { },
  getCellProperties: function(index, column, properties) { },
  getColumnProperties: function(column, properties) { },
  isContainer: function(index) { return false; },
  isContainerOpen: function(index) { return false; },
  isContainerEmpty: function(index) { return false; },
  isSeparator: function(index) { return false; },
  getParentIndex: function(index) { return -1; },
  hasNextSibling: function(parentIndex, index) { return false; },
  getLevel: function(index) { return 0; },
  getProgressMode: function(index, column) { },
  getCellValue: function(index, column) { },
  toggleOpenState: function(index) { },
  selectionChanged: function() { },
  cycleCell: function(row, column) { },
  isEditable: function(index, column) { return false; },
  isSelectable: function(index, column) { return false; },
  setCellValue: function(index, column, value) { },
  setCellText: function(index, column, value) { },
  performAction: function(action) { },
  performActionOnRow: function(action, index) { },
  performActionOnCell: function(action, index, column) { }
};

